import { PageViewport, PDFViewer } from 'pdfjs-dist/web/pdf_viewer';
import { MouseEvent } from 'react';

export interface TLTWH {
	left: number;
	top: number;
	width: number;
	height: number;
}

export type TContent = {
	text?: string;
	image?: string;
};

export type THighlightContent = {
	content: TContent;
};

export type TComment = {
	text: string;
	emoji: string;
};
export type THighlightComment = {
	comment: TComment;
};

export type TNewHighlight = {
	position: TLTWH;
	pageNumber: number;
};

export type IHighlightType = 'table' | 'cell' | 'page' | 'document' | 'form' | 'word';

export type THighlight = {
	id: string;
	selectedColor: string;
	selectedBorderColor?: string;
	type: IHighlightType;
	unselectedBorderColor?: string;
	unselectedColor?: string;
	label?: string;
	groupId?: string;
} & TNewHighlight;

export interface TPDFJSDocument {
	destroy: () => void;
	numPages: number;

	getPage(number: number): any;
}

export interface TPDFJSLinkService {
	setDocument: (document: any) => void;
	setViewer: (viewer: PDFViewer) => void;
}

export interface TPDFJS {
	TextLayerBuilder: {
		prototype: {
			_bindMouse: () => void;
		};
	};
	PDFViewer: (options: any) => PDFViewer;
	PDFLinkService: () => TPDFJSLinkService;
	getDocument: (url: string) => Promise<TPDFJSDocument>;
	disableWorker: boolean;
}

export type THighlightItem<THT> = Omit<THT, 'position'> & { position: TLTWH };

export type THighLightItemProps = {
	onClick?: (itemId: string) => void;
	onDoubleClick?: (itemId: string) => void;
	onMouseOver?: () => void;
	onMouseOut?: () => void;
};

export interface PdfHighlighterProps<THT> {
	highlights: Array<THT>;
	pdfDocument: TPDFJSDocument;
	pdfScaleValue: string;
	removePageBorders?: boolean;
	onChangeActivePage(page: number): void;
	activePage: number;
	highlightAreaProps?: THighLightItemProps;
}

export interface SinglePdfRenderProps {
	pdfDocument: TPDFJSDocument;
	pageNumber: number;
	highlights?: Array<THighlight>;
	highlightAreaProps?: THighLightItemProps;
	activeHighlightId?: string;
	focusHighlightId?: string;
	zoomValue?: number;
	rotationValue?: number;
	onClick?(evt: MouseEvent<HTMLDivElement>): void;
	onAfterPageRendered?(): void;
}

export interface PdfLoaderProps {
	data?: Blob;
	url?: string;
	beforeLoad: any;
	errorMessage?: any;
	children: (pdfDocument: TPDFJSDocument) => any;
	onError?: (error: Error) => void;
	cMapUrl?: string;
	cMapPacked?: boolean;
}

export interface PDFRenderRef {
	viewport: PageViewport;
	container: HTMLDivElement;
}

export interface PDFPage {
	div: HTMLDivElement;
	id: number;
}

export { PageViewport, PDFViewer };
